/*
   Copyright (c) 1991 Jyh-Shing Roger Jang, Dept of EECS, U.C. Berkeley
   BISC (Berkeley Initiative on Soft Computing) group
   jang@eecs.berkeley.edu
   Permission is granted to modify and re-distribute this code in any manner
   as long as this notice is preserved.  All standard disclaimers apply.
*/

/*
   ANFISLAB: Toolbox de ANFIS (ADAPTATIVE-NETWORK-BASED-FUZZY INFERENCE SYSTEM) para SCILAB
   Creado Por Andres G. Avendao Cabezas., Universidad Central de Venezuela (andrewg.avendano@gmail.com),
  Agosto de 2009, con adaptaciones del codigo ANFIS de dominio publico de Jyh-shing Roger Jang
  (http://www.cs.nthu.edu.tw/~jang/publication.htm).
   Este programa fue probado en SCILAB version 5.1.1, PC Intel Core(TM) Duo 2,1Ghz, corriendo Linux 2.6.27.7
*/

#include "anfis.h"
#include <stdio.h>
#include <math.h>

static int **config;

void
get_config()
{
	int i, j;
	int connected();
   int tmp;
config = (int **)create_matrix(Node_n, Node_n, sizeof(int));
	for (i = 0; i < Node_n; i++){
		for (j = 0; j < Node_n; j++){
         tmp = connected(i, j);
			config[i][j] = tmp;
         }
	}
}

/* return 1 if there is a link from node i to node j */
int
connected(i, j)
int i, j;
{
	int which_layer();
	void other_rep();
	int layer1, layer2;
	int group;
	int position;
	static int *rep;
	static initialization;
	if (i >= j)
		return(0);
	layer1 = which_layer(i);
	layer2 = which_layer(j);
	if ((layer2 - layer1 != 1) && layer1 != 0)
		return(0);
	switch(layer1) {
		case 0: 
			/* connections from layer 0 to layer 1 */
			if (between(In_n + i*Mf_n, j, In_n + (i+1)*Mf_n - 1)) 
				return(1);
			/* connections from layer 0 to layer 5 */
			if (between(In_n + In_n*Mf_n + 2*Rule_n, j, 
				    In_n + In_n*Mf_n + 3*Rule_n- 1))
				return(1);
			break;
		case 1:
			if (initialization != 7151373){ 
				rep = (int *)calloc(In_n, sizeof(int));
				initialization = 7151371;
			}
			group = (i - In_n)/Mf_n;
			position = (i - In_n) % Mf_n;
			other_rep(rep, j - In_n - In_n*Mf_n);
			if (rep[group] == position)
				return(1);
			break;
		case 2:
			/* layer 2 and 3 are fully connected */
			return(1);
			break;
		case 3:
			if (j - i == Rule_n)
				return(1);
			break;
		case 4:
			/* layer 4 and 5 are fully connected */
			return(1);
			break;
		case 5:
			return(0);
			break;
		default:
			printf("Error in layer!\n");
	}
	return(0);
}

/* other representation of j using mf_n as the base */
void
other_rep(rep, j)
int j;
int *rep;
{
	int i;
	for (i = 0; i < In_n; i++) {
		rep[In_n - i - 1] = j % Mf_n;
		/* j = (int)floor((double)(j/mf_n)); */
		j = j/Mf_n;
	}
}
		
/* return the layer of given node index */
int 
which_layer(index)
int index;
{
	int between();
	if (between(0, index, In_n - 1))
		return(0);
	if (between(In_n, index, In_n + In_n*Mf_n - 1))
		return(1);
	if (between(In_n + In_n*Mf_n, index, In_n + In_n*Mf_n + Rule_n- 1))
		return(2);
	if (between(In_n + In_n*Mf_n + Rule_n, index, In_n + In_n*Mf_n + 2*Rule_n- 1))
		return(3);
	if (between(In_n + In_n*Mf_n + 2*Rule_n, index, In_n + In_n*Mf_n + 3*Rule_n- 1))
		return(4);
	if (index = In_n + In_n*Mf_n + 3*Rule_n)
		return(5);
	printf("Error in which_layer!\n");
	exit(1);
}

/* return 1 if x is between l and u, that is, l <= x <= u */
int
between(l, x, u)
int l, x, u;
{
/*      printf("l%d \t x%d \t u%d \n",l,x,u);*/
	if ((l <= x) && (x <= u))
		return(1);
	else 
		return(0);
}

void
build_anfis()
{
	void build_layer();
	NODE_LIST_T *build_node_list();
	int i;

	build_layer(0, In_n, 0, 0, 0); 
	build_layer(1, In_n*Mf_n, In_n, 3, 1); 
	build_layer(2, Rule_n, In_n+In_n*Mf_n, 0, 2); 
	build_layer(3, Rule_n, In_n+In_n*Mf_n+Rule_n, 0, 3); 
	build_layer(4, Rule_n, In_n+In_n*Mf_n+2*Rule_n, In_n + 1, 4); 
	build_layer(5, 1, In_n+In_n*Mf_n+3*Rule_n, 0, 5); 

	for (i = 0; i < Node_n; i++) {
		node_p[i]->fan_in  = build_node_list(0, i);
		node_p[i]->fan_out = build_node_list(1, i);
	}
}

/* Build a node list of layer-th layer, with n node, 
   starting at index index, and each
   node has parameter_n parameters and node function 
   function[function_index]. */
void
build_layer(layer, n, index, parameter_n, function_index)
int layer, n, index, parameter_n, function_index;
{
	PARAMETER_LIST_T *build_parameter_list();
	int i;
	NODE_T *q;

	if (n == 0)
		printf("Possible error in build_layer!\n");

	for (i = 0; i < n; i++) {
		q = (NODE_T *) malloc(sizeof (NODE_T));
		node_p[i + index] = q;
		q->index = i + index;
		q->layer = layer;
		q->local_index = i;
		q->parameter_n = parameter_n;
		q->function_index = function_index;
		q->parameter = build_parameter_list(parameter_n);
	}
}

/* Build a parameter list with length n. */
PARAMETER_LIST_T *
build_parameter_list(n)
int n;
{
	PARAMETER_LIST_T *head, *p, *q;
	int i;

	if (n < 0)
		exit1("Error in build_parameter_list()!");

	if (n == 0)
		return(NULL);

	head = (PARAMETER_LIST_T *) malloc(sizeof (PARAMETER_LIST_T));
	p = head;
	for (i = 1; i < n; i++){
		q = (PARAMETER_LIST_T *) malloc(sizeof (PARAMETER_LIST_T));
		p->next = q;
		p = q;
	}
	p->next = NULL;
	return(head);
}

/* type == 0 --> build node list along column n of matrix config.
   type == 1 --> build node list along row n of matrix config.     */
NODE_LIST_T *
build_node_list(type, n)
int type, n;
{
	NODE_LIST_T *p, *q, *dummy;
	int i;

	p = (NODE_LIST_T *) malloc(sizeof (NODE_LIST_T));
	dummy = p;
	dummy->next = NULL;
	if (type == 0) 
		for (i = 0; i < Node_n; i++)
			if (config[i][n]){
				q = (NODE_LIST_T *) malloc(sizeof (NODE_LIST_T));
				q->content = node_p[i];
				p->next = q;
				p = q;
			}
	if (type == 1) 
		for (i = 0; i < Node_n; i++)
			if (config[n][i]){
				q = (NODE_LIST_T *) malloc(sizeof (NODE_LIST_T));
				q->content = node_p[i];
				p->next = q;
				p = q;
			}
	p->next = NULL;
	q = dummy;
	dummy = dummy->next;
	free(q);
	return(dummy);
}

/* set parameter mode: fixed = 1 --> fixed parameter;
		       fixed = 0 --> modifiable parameter */
int
set_parameter_mode()
{
	int i, modifiable_p_count = 0;
	PARAMETER_LIST_T *p;

	for (i = 0; i < Node_n; i++) {
		if (node_p[i]->parameter == NULL)
			continue;
		for (p = node_p[i]->parameter; p != NULL; p = p->next) {
			p->fixed = 0;
			modifiable_p_count++;
		}
	}
	printf("Modifiable parameters: %d\n", modifiable_p_count);
	return(modifiable_p_count);
}
