/*
   Copyright (c) 1991 Jyh-Shing Roger Jang, Dept of EECS, U.C. Berkeley
   BISC (Berkeley Initiative on Soft Computing) group
   jang@eecs.berkeley.edu
   Permission is granted to modify and re-distribute this code in any manner
   as long as this notice is preserved.  All standard disclaimers apply.
*/

/*
   ANFISLAB: Toolbox de ANFIS (ADAPTATIVE-NETWORK-BASED-FUZZY INFERENCE SYSTEM) para SCILAB
   Creado Por Andres G. Avendao Cabezas., Universidad Central de Venezuela (andrewg.avendano@gmail.com),
  Agosto de 2009, con adaptaciones del codigo ANFIS de dominio publico de Jyh-shing Roger Jang
  (http://www.cs.nthu.edu.tw/~jang/publication.htm).
   Este programa fue probado en SCILAB version 5.1.1, PC Intel Core(TM) Duo 2,1Ghz, corriendo Linux 2.6.27.7
*/
#define __USE_DEPRECATED_STACK_FUNCTIONS__
#include "anfis.h"
#include <sciprint.h>
#include <stack-c.h>
/* calculate node outputs from node 'from' to node 'to' */
void
calculate_output(from, to)
int from, to;
{
	double input(), funsel(), multiply(); 
	double normalize(), consequent(), sumatoria();
	static double (*function[6])() = {input, funsel, multiply, 
	                          normalize, consequent, sumatoria};
	int i;
	int function_index;
	if ((from > to) || (from < In_n) || (to >= Node_n))
		exit1("Error in calculate_output!");
	for (i = from; i <= to; i++) {
		function_index= node_p[i]->function_index;
		node_p[i]->value = (*function[function_index])(i);
	}
}

double
input(node_index)
int node_index;
{
	printf("This shouldn't have been called!\n");
	return(0);
}

double
multiply(node_index)
int node_index;
{
	double product = 1.0;
	NODE_LIST_T *p;

	NODE_LIST_T *arg_p = node_p[node_index]->fan_in;
	PARAMETER_LIST_T *para_p = node_p[node_index]->parameter;
	if (para_p != NULL)
		exit1("Error in multiply!");
	for (p = arg_p; p != NULL; p = p->next)
		product *= p->content->value;
	//sciprint("Layer 2:   %f\n",product);
	return(product); 
}
 
double 
normalize(node_index)
int node_index;
{
	NODE_LIST_T *arg_p = node_p[node_index]->fan_in;
	PARAMETER_LIST_T *para_p = node_p[node_index]->parameter;
	int i;
	double denom = 0;
	NODE_LIST_T *p;

	if (para_p != NULL)
		exit1("Error in normalize!");

	//sciprint("printing a double:   %f\n", denom);
	for (p = arg_p; p != NULL; p = p->next){
		denom += p->content->value;
	}

	p = arg_p;
	for (i = 0; i < node_p[node_index]->local_index; i++) 
		p = p->next; 

	if (denom == 0)
       denom=1;
/*		exit1("Error in normalize!");*/
	//sciprint("printing a double:   %f\t %f\n",p->content->value, denom );
	//sciprint("Layer 3:   %f\n",p->content->value/denom);
	return(p->content->value/denom);

	
	//return(p->content->value);

}

double 
consequent(node_index)
int node_index;
{
	NODE_LIST_T *arg_p = node_p[node_index]->fan_in;
	PARAMETER_LIST_T *para_p = node_p[node_index]->parameter;
	int i;
	double x, a, total = 0;
	for (i = 0; i < In_n + 1; i++) {
		x = arg_p->content->value;
		a = para_p->content;
		if (i == In_n)
			break;
		total += x*a;
		arg_p = arg_p->next;
		para_p = para_p->next;
	}
	//sciprint("printing a double:   %f\t %f\n",x, a );
	//sciprint("Layer 4:   %f\n",x*(total + a));
	return(x*(total + a));
	
	/*
	double a1, b1, c1, wn1, x11, x21;
	x11 = arg_p->content->value;
	x21 = arg_p->next->content->value;
	wn1 = arg_p->next->next->content->value;
	a1 = para_p->content;
	b1 = para_p->next->content;
	c1 = para_p->next->next->content;
	return((a1*x11 + b1*x21 + c1)*wn1);
	*/
}

double 
sumatoria(node_index)
int node_index;
{
	NODE_LIST_T *arg_p = node_p[node_index]->fan_in;
	NODE_LIST_T *t;

	double total = 0;
	if (arg_p == NULL)
		exit1("Error! Given pointer is NIL!");

	for (t = arg_p; t != NULL; t = t->next)
		total += t->content->value;

	//sciprint("Layer 5:   %f\n",total);
	return(total);
}
/*modificaciones de andres avendaño*/

double
funsel(node_index)
int node_index;
{
	double bell_1(),bell_2(),trapezoidal(),triangular(); 
	switch (funper) {
		case 1:{
			//sciprint("Layer 1:   %f\n",bell_1(node_index));
				return(bell_1(node_index));}
		case 2:{
			//sciprint("Layer 1:   %f\n",bell_2(node_index));
				return(bell_2(node_index));}
		case 3: {
			//sciprint("Layer 1:   %f\n",trapezoidal(node_index));
				return(trapezoidal(node_index));}
		case 4: {
			//sciprint("Layer 1:   %f\n",triangular(node_index));
				return(triangular(node_index));}
		default:
			exit1("Error en funper, debe asignar un entero entre 1 y 4!");
	}
}
/* membership function = exp(-(pow(pow((x - c)/a, 2),b))*/
double 
bell_1(node_index)
int node_index;
{
	NODE_LIST_T *arg_p = node_p[node_index]->fan_in;
	PARAMETER_LIST_T *para_p = node_p[node_index]->parameter;
	double c, a, b, x;
	double tmp1, tmp2;

	x = arg_p->content->value;
	a = para_p->content;
	b = para_p->next->content;
	c = para_p->next->next->content;
	if (a == 0)
		exit1("Error in bell_1!");
	tmp1 = (x - c)/a;
	tmp2 = tmp1 == 0 ? 0 : pow(pow(tmp1, 2.0), b);
	return(1/(exp(tmp2))); 
}
/* membership function = 1/(1+pow((x - c)/a, 2*b)) */
//f(x,a,b,c)=1/(1+abs((x-c)/a)^(2*b))
double 
bell_2(node_index)
int node_index;
{
	NODE_LIST_T *arg_p = node_p[node_index]->fan_in;
	PARAMETER_LIST_T *para_p = node_p[node_index]->parameter;
	double c, a, b, x;
	double tmp1, tmp2;
	
	x = arg_p->content->value;
	a = para_p->content;
	b = para_p->next->content;
	c = para_p->next->next->content;
/*printf("a %10f.\t b %10fd.\t c %10f.\n", a, b, c);*/
	if (a == 0)
		exit1("Error in bell_2!");
	tmp1 = (x - c)/a;
	tmp2 = tmp1 == 0 ? 0 : pow(pow(tmp1, 2.0), b);
	return(1/(1+ tmp2)); 
}
/* membership function = trapezoidal */
double 
trapezoidal(node_index)
int node_index;
{
	NODE_LIST_T *arg_p = node_p[node_index]->fan_in;
	PARAMETER_LIST_T *para_p = node_p[node_index]->parameter;
	double c, a, b, x;
	double tmp1, tmp2;

	x = arg_p->content->value;
	a = para_p->content;
	b = para_p->next->content;
	c = para_p->next->next->content;
	if (a == 0)
		exit1("Error en trapezoidal!");
	tmp1 = c+b/2+1/a;
	tmp2 = c-b/2-1/a;
      if ( (x >= tmp1) || (x <= tmp2) )
	      return(0.0); 
      else if ( (x < (c+b/2)) && (x > (c-b/2) ) )
      	return(1); 
      else if ( x >= (c + b/2)){
      	return(a*(tmp1-x)); 
      }
      else if ( x <= (c - b/2)){
      	return(a*(x-tmp2));
      }
      else
         exit1("Error en trapezoidal!");
}

double 
triangular(node_index)
int node_index;
{
	NODE_LIST_T *arg_p = node_p[node_index]->fan_in;
	PARAMETER_LIST_T *para_p = node_p[node_index]->parameter;
	double c, a, x;
	double tmp1, tmp2;

	

	x = arg_p->content->value;
	a = para_p->content;
	c = para_p->next->next->content;
	if (a == 0)
		exit1("Error en triangular1!");
	tmp1 = c+1/a; // hi
	tmp2 = c-1/a; //low
      
	if ( (x >= tmp1) || (x <= tmp2) )
	      return(0.0); 
    else if ( x == c )
      	return(1); 
    else if ( x > (c))
		  return(a*(c-x)+1); 
    else if ( x < (c))
		  return(a*(x-c)+1);
    else
         exit1("Error en triangular2!");


	//if ( (x >= tmp1) || (x <= tmp2) )
	//      return(0.0); 
 //     else if ( (x < (c) ) && (x > (c) ) )
 //     	return(1); 
 //     else if ( x >= (c)){
	//	  return(a*(c-x)+1); 
 //     	//return(a*(tmp1-x)); 
 //     }
 //     else if ( x <= (c)){
	//	  return(a*(c+x)+1);
 //     	//return(a*(x-tmp1));
 //     }
 //     else
 //        exit1("Error en triangular2!");


}


//double 
//triangular(node_index)
//int node_index;
//{
//	NODE_LIST_T *arg_p = node_p[node_index]->fan_in;
//	PARAMETER_LIST_T *para_p = node_p[node_index]->parameter;
//	double c, b, a, x;
//	double tmp1, tmp2;
//
//	x = arg_p->content->value;
//	a = para_p->content;
//	b = para_p->next->content;
//	c = para_p->next->next->content;
//	//if (a == 0)
//	//	exit1("Error en triangular!");
//	tmp1 = (x-b)/(b-a);
//	tmp2 = (c-x)/(c-b);
//      if ( (x <= a) || (x >= c) )
//	      return(0.0); 
//      else if ( (x > (a) ) && (x < (b) ) )
//      	return(tmp1); 
//      else if ( x == (b)){
//      	return(1); 
//      }
//      else if ( (x > (b) ) && (x < (c) )){
//      	return(tmp2);
//      }
//      else
//         exit1("Error en triangular!");
//}
