/*
   Copyright (c) 1991 Jyh-Shing Roger Jang, Dept of EECS, U.C. Berkeley
   BISC (Berkeley Initiative on Soft Computing) group
   jang@eecs.berkeley.edu
   Permission is granted to modify and re-distribute this code in any manner
   as long as this notice is preserved.  All standard disclaimers apply.
*/
/*
   ANFISLAB: Toolbox de ANFIS (ADAPTATIVE-NETWORK-BASED-FUZZY INFERENCE SYSTEM) para SCILAB
   Creado Por Andres G. Avendao Cabezas., Universidad Central de Venezuela (andrewg.avendano@gmail.com),
  Agosto de 2009, con adaptaciones del codigo ANFIS de dominio publico de Jyh-shing Roger Jang
  (http://www.cs.nthu.edu.tw/~jang/publication.htm).
   Este programa fue probado en SCILAB version 5.1.1, PC Intel Core(TM) Duo 2,1Ghz, corriendo Linux 2.6.27.7
*/

#include "anfis.h"
#include "misc_def.h"

NODE_T **node_p;
int In_n, Mf_n, Rule_n, Node_n, funper;
void
   hyonline(In_n, Mf_n, training_data_n, funper, step_size, abc, pqr, training_data_matrix, trn_evol_error)
      int training_data_n;
      double step_size;
      double **abc, **pqr;
      double **training_data_matrix;
   	double **trn_evol_error;
{
	int i, j, k, debug;
   	int parameter_n, first; 
  	double increase_rate, decrease_rate;/*agregado*/
	double min_RMSE = pow(2.0, 31.0) - 1;
	double *kalman_data, *kalman_parameter;
 	double *trn_rmse_error;
	double de_dout, target, lambda;
	double **layer_1_to_3_output, **A,**B, **X;
	double **training_data_pointer;
	double *parameter_array, *step_size_array, *anfis_output;
	double *trn_error;
   double trn_evol[training_data_n], step_evol[training_data_n];
   decrease_rate = 0.9;
   increase_rate = 1.1;
   lambda=0.99; 
  debug=0;
	Rule_n = (int)pow((double)Mf_n, (double)In_n);
	Node_n = In_n + In_n*Mf_n + 3*Rule_n + 1;
	first = In_n + In_n*Mf_n + Rule_n; 
	/* allocate matrices */
	node_p = (NODE_T **)create_array(Node_n, sizeof(NODE_T *));
	training_data_pointer = (double **)create_matrix
			(training_data_n, In_n + 1, sizeof(double));
	layer_1_to_3_output = (double **)create_matrix
		(training_data_n, In_n*Mf_n + 2*Rule_n, sizeof(double));

	A = (double **)create_matrix(training_data_n, Rule_n, sizeof(double));
	B = (double **)create_matrix(training_data_n, 1, sizeof(double));
	X = (double **)create_matrix(Rule_n, 1, sizeof(double));

	trn_rmse_error = (double *)calloc(training_data_n, sizeof(double));
	kalman_data = (double *)
		      calloc((In_n + 1)*Rule_n + 1, sizeof(double));
	kalman_parameter = (double *)calloc
			   ((In_n + 1)*Rule_n, sizeof(double));
	step_size_array = (double *)calloc(training_data_n, sizeof(double));
	anfis_output = (double *)calloc(training_data_n, sizeof(double));
	trn_error = (double *)calloc(4, sizeof(double));

	get_config();
	build_anfis();
	parameter_n = set_parameter_mode();
	parameter_array = (double *)calloc(parameter_n, sizeof(double));
	get_parameter(abc,pqr);

   get_data(training_data_matrix, training_data_n, training_data_pointer);
	printf("epochs \t trn error \t step error\n");
	printf("------ \t --------- \t ----------\n");
	for (i = 0; i < training_data_n; i++) {
		step_size_array[i] = step_size;
		for (j = 0; j < training_data_n; j++) {
			put_input_data(j, training_data_pointer);
			/* get node outputs from layer 1 to layer 3 */
			calculate_output(In_n, In_n + In_n*Mf_n + 2*Rule_n - 1);
			/* put outputs of layer 1 to 3 
			   into layer_1_to_3_output */
			for (k = 0; k < Mf_n*In_n + 2*Rule_n; k++)
				layer_1_to_3_output[j][k] 
					= node_p[k + In_n]->value;

		for (k = first; k < first + Rule_n; k++)
			A[j][k-first] = (node_p[k]->value);
			B[j][0] = training_data_matrix[j][In_n]; 

		}
		LSE_kalman(A, B, X, training_data_n, Rule_n, 1, lambda);
		for (j = 0; j < Rule_n; j++) 
		kalman_parameter[j]=X[j][0];
		clear_de_dp();
		for (j = 0; j < training_data_n; j++) {
			put_input_data(j, training_data_pointer);
			/* get output of layer 1 to 3 
			   from layer_1_to_3_output */
			for (k = 0; k < Mf_n*In_n + 2*Rule_n; k++){
				node_p[k + In_n]->value 
					= layer_1_to_3_output[j][k]; 
         }
			/* calculate outputs of layer 4 and 5 */
			calculate_output(In_n + In_n*Mf_n + 2*Rule_n, Node_n - 1); 
			anfis_output[j] = node_p[Node_n - 1]->value;
			target = training_data_matrix[j][In_n]; 
			de_dout = -2*(target - node_p[Node_n-1]->value);
			calculate_de_do(de_dout);
			update_de_dp();
		}
		training_error_measure(training_data_pointer, anfis_output,
			training_data_n, trn_error);
		trn_rmse_error[i] = trn_error[0];
			printf("%4d \t %lf\n", i+1,
				trn_error[ERROR_TYPE]);
		if (trn_rmse_error[i] < min_RMSE) {
			min_RMSE = trn_rmse_error[i];
			record_parameter(parameter_array);
		}
		/* update parameters in 1st layer */
		update_parameter(1, step_size);

		update_step_size(trn_rmse_error, i, &step_size, 
			decrease_rate, increase_rate);
	}
	printf("Minimal training RMSE = %lf\n", min_RMSE);
	restore_parameter(parameter_array);
	write_parameter(abc,pqr);
	write_array(trn_rmse_error, training_data_n, trn_evol);
	write_array(step_size_array, training_data_n, step_evol);
   for (j = 0; j < training_data_n; j++) {
   trn_evol_error[j][0]=j+1;
      trn_evol_error[j][1]=trn_rmse_error[j];
         trn_evol_error[j][2]=step_size_array[j];
   }
}
